import {Skill, Stat} from "./datatypes.ts";
import {getSkills} from "./skills.ts";

export class PlayerProgress {
  #stats: Record<Stat, number>
  #blood: number
  #skillsLearned: number[]  // use the raw ID representation for indexOf
  #untrimmedSkillsAvailable: Skill[]

  constructor() {
    this.#stats = {
      AGI: 10,
      INT: 10,
      CHA: 10,
      PSI: 10,
    };
    this.#blood = 0;
    this.#skillsLearned = [];
    this.#untrimmedSkillsAvailable = []

    this.refill();
  }

  refill() {
    this.#blood = 2000;

    let learnableSkills = [];  // TODO: Also include costing info
    for (let skill of getSkills().getAllAvailableSkills().values()) {
      if (this.#canBeAvailable(skill)) {
        learnableSkills.push(skill);
      }
    }

    this.#untrimmedSkillsAvailable = learnableSkills
  }

  hasLearned(skill: Skill) {
    return this.#skillsLearned.indexOf(skill.id) !== -1;
  }

  learnSkill(skill: Skill) {
    if (this.#skillsLearned.indexOf(skill.id) != -1) {
      return
    }
    this.#skillsLearned.push(skill.id);

    // remove entries for that skill
    let skills2 = [];
    for (let entry of this.#untrimmedSkillsAvailable.values()) {
      if (entry.id == skill.id) { continue; }
      skills2.push(entry);
    }
    this.#untrimmedSkillsAvailable = skills2;
  }

  #canBeAvailable(skill: Skill) {
    // make sure we haven't learned this skill already
    if (this.hasLearned(skill)) {
      return false;
    }

    let data = getSkills().get(skill);

    // make sure the prereqs are met
    for (let prereq of data.prereqs.values()) {
      if (!this.hasLearned(prereq)) {
        return false
      }
    }

    // ok, we're good!!
    return true;
  }

  add(stat: Stat, amount: number) {
    if (amount != Math.floor(amount)) {
      throw `stat increment must be integer: ${amount}`
    }
    if (amount <= 0) {
      throw `stat increment must be >0: ${amount}`
    }
    this.#stats[stat] += amount;
  }

  getStat(stat: Stat): number {
    return this.#stats[stat]
  }

  getBlood(): number {
    return Math.floor(Math.max(this.#blood, 0));
  }

  spendBlood(amt: number) {
    this.#blood -= amt;
  }

  getAvailableSkills(): Skill[] {
    // Sort by cost, then by name, then trim down to first 6
    let skillsAvailable = [...this.#untrimmedSkillsAvailable];
    skillsAvailable.sort((a, b) => {
      return getSkills().computeCost(a) - getSkills().computeCost(b)
    });
    skillsAvailable.sort((a, b) => {
      let name1 = getSkills().get(a).profile.name;
      let name2 = getSkills().get(b).profile.name;

      if (name1 < name2) { return -1; }
      if (name1 > name2) { return 1; }
      return 0;
    });
    return skillsAvailable.slice(0, 6)
  }
}

let active: PlayerProgress = new PlayerProgress();

export function getPlayerProgress(): PlayerProgress {
  return active
}